@@ -2,38 +2,25 @@
 //! A library for fast, offline reverse geocoding. The location data are from [GeoNames](http://www.geonames.org/).
 //!
 //! # Usage
-//! First, add this to your Cargo.toml
-//!
-//! ```toml
-//! [dependencies]
-//! reverse_geocoder = "^1.0.1"
 //! ```
-//!
-//! Next:
-//!
-//! ```
-//! use reverse_geocoder::{Locations, ReverseGeocoder};
+//! use reverse_geocoder::{Locations, ReverseGeocoder, SearchResult};
 //!
 //! fn main() {
 //!     let loc = Locations::from_memory();
 //!     let geocoder = ReverseGeocoder::new(&loc);
-//!     let y = geocoder.search(&[45.0, 54.0]).expect("Search error.");
-//!     let pair = y.get(0).expect("No results.");
-//!     println!("Distance {}", pair.0);
-//!     println!("Record {}", pair.1);
+//!     let coords = (40.7831, -73.9712);
+//!     let search_result = geocoder.search(coords).unwrap();
+//!     println!("Distance {}", search_result.distance);
+//!     println!("Record {}", search_result.record);
 //! }
 //!```
 
-#[macro_use]
-extern crate failure;
-#[macro_use]
-extern crate serde_derive;
-
-use kdtree::{distance::squared_euclidean, ErrorKind, KdTree};
+use kdtree::{distance::squared_euclidean, KdTree};
 // use time::Instant;
-use failure::Error;
+use serde_derive::{Serialize, Deserialize};
+use std::error;
 use std::fmt;
-use std::path::PathBuf;
+use std::path::Path;
 
 /// A parsed location.
 #[derive(Debug, Clone, RustcDecodable, RustcEncodable, Serialize, Deserialize)]
@@ -52,6 +39,14 @@ pub struct Record {
     pub admin3: String,
 }
 
+/// Search result from querying a lat/long.
+pub struct SearchResult<'a> {
+    /// Distance away from given lat/long.
+    pub distance: f64,
+    /// Closest place information.
+    pub record: &'a Record,
+}
+
 /// A set of location records.
 pub struct Locations {
     records: Vec<([f64; 2], Record)>,
@@ -72,26 +67,18 @@ impl Locations {
     }
 
     /// Supply your own path to a CSV file.
-    pub fn from_path(path: Option<PathBuf>) -> Result<Locations, Error> {
-        // let start = Instant::now();
+    pub fn from_path<P: AsRef<Path>>(file_path: P) -> Result<Locations, Box<dyn error::Error>> {
+        // let start_load = Instant::now();
         let mut records = Vec::new();
 
-        let path = match path {
-            Some(path) => path,
-            None => PathBuf::from("cities.csv"),
-        };
-
-        let reader = quick_csv::Csv::from_file(path)?.has_header(true);
+        let reader = quick_csv::Csv::from_file(file_path)?.has_header(true);
 
         for read_record in reader {
             let record: Record = read_record?.decode()?;
             records.push(([record.lat, record.lon], record));
         }
 
-        // let end = Instant::now();
-
-        // println!("{} ms to load cities.csv", (end - start).whole_milliseconds());
-
+        // eprintln!("{} ms to load csv", start_load.elapsed().whole_milliseconds());
         Ok(Locations { records: records })
     }
 }
@@ -120,9 +107,27 @@ impl<'a> ReverseGeocoder<'a> {
         // println!("{} ms to build the KdTree", (end - start).whole_milliseconds());
     }
 
-    /// Search for the closest record to a given lat/long. Returns Result<Vec<(distance, record)>>.
-    pub fn search(&self, loc: &[f64; 2]) -> Result<Vec<(f64, &&Record)>, ErrorKind> {
-        self.tree.nearest(loc, 1, &squared_euclidean)
+    /// Search for the closest record to a given (latitude, longitude). Non-finite numbers will always return None.
+    pub fn search(&self, loc: (f64, f64)) -> Option<SearchResult> {
+        let nearest = match self.tree.nearest(&[loc.0, loc.1], 1, &squared_euclidean) {
+            Ok(nearest) => nearest,
+            Err(error) => match error {
+                kdtree::ErrorKind::WrongDimension => {
+                    panic!("Internal error, kdtree::ErrorKind::WrongDimension should never occur")
+                }
+                kdtree::ErrorKind::NonFiniteCoordinate => return None,
+                kdtree::ErrorKind::ZeroCapacity => {
+                    panic!("Internal error, kdtree::ErrorKind::ZeroCapacity should never occur")
+                }
+            },
+        };
+        match nearest.get(0) {
+            Some(nearest) => Some(SearchResult {
+                distance: nearest.0,
+                record: nearest.1,
+            }),
+            None => None,
+        }
     }
 }
 
@@ -139,32 +144,60 @@ impl fmt::Display for Record {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use failure::Error;
 
     #[test]
-    fn it_finds_3_places() {
+    fn it_finds_4_places() {
         let loc = Locations::from_memory();
         let geocoder = ReverseGeocoder::new(&loc);
-        let y = geocoder.search(&[44.962786, -93.344722]);
-        assert_eq!(y.is_ok(), true);
-        let slp = y.unwrap();
 
-        assert_eq!(slp.get(0).unwrap().1.name, "Saint Louis Park");
+        let slp = geocoder.search((40.7831, -73.9712)).unwrap();
+        assert_eq!(slp.record.name, "Manhattan");
+
+        let slp = geocoder.search((44.962786, -93.344722)).unwrap();
+
+        assert_eq!(slp.record.name, "Saint Louis Park");
 
         // [44.894519, -93.308702] is 60 St W @ Penn Ave S, Minneapolis, Minnesota; however, this is physically closer to Richfield
-        let mpls = geocoder.search(&[44.894519, -93.308702]).unwrap();
-        assert_eq!(mpls.get(0).unwrap().1.name, "Richfield");
+        let mpls = geocoder.search((44.894519, -93.308702)).unwrap();
+        assert_eq!(mpls.record.name, "Richfield");
 
         // [44.887055, -93.334204] is HWY 62 and Valley View Road, whish is in Edina
-        let edina = geocoder.search(&[44.887055, -93.334204]).unwrap();
-        assert_eq!(edina.get(0).unwrap().1.name, "Edina");
+        let edina = geocoder.search((44.887055, -93.334204)).unwrap();
+        assert_eq!(edina.record.name, "Edina");
     }
 
     #[test]
-    fn locations_from_path() -> Result<(), Error> {
-        let loc = Locations::from_path(Some("./cities.csv".into()))?;
-        ReverseGeocoder::new(&loc);
-
+    fn it_loads_locations_from_a_path() -> Result<(), Box<dyn error::Error>> {
+        let loc = Locations::from_path("./cities.csv")?;
+        let geocoder = ReverseGeocoder::new(&loc);
+        let search_result = geocoder.search((45.0, 54.0));
+        assert!(search_result.is_some());
         Ok(())
     }
+
+    #[test]
+    fn it_loads_locations_from_a_nearly_blank_file() -> Result<(), Box<dyn error::Error>> {
+        let loc = Locations::from_path("./nearly-blank.csv")?;
+        let geocoder = ReverseGeocoder::new(&loc);
+        let search_result = geocoder.search((45.0, 54.0));
+        assert!(search_result.is_none());
+        Ok(())
+    }
+
+    #[test]
+    fn it_loads_locations_from_a_blank_file() -> Result<(), Box<dyn error::Error>> {
+        let loc = Locations::from_path("./blank.csv")?;
+        let geocoder = ReverseGeocoder::new(&loc);
+        let search_result = geocoder.search((45.0, 54.0));
+        assert!(search_result.is_none());
+        Ok(())
+    }
+
+    #[test]
+    fn it_returns_none_given_an_infinite_coordinate() {
+        let loc = Locations::from_memory();
+        let geocoder = ReverseGeocoder::new(&loc);
+        let search_result = geocoder.search((std::f64::INFINITY, 54.0));
+        assert!(search_result.is_none());
+    }
 }
